
import pylab
import matplotlib.pyplot as plt
import matplotlib.animation as animation
import matplotlib.patches as mpatches
import matplotlib as mpl
import riemann.function

from threading import Thread

"""
Constants
"""
SHOW_REAL = True
SHOW_IMAG = True

VIEW_LINE_REAL = .5

X_INCREMENT = .025
X_MIN = -20.0
X_MAX = 20.0
X_DIFF = X_MAX - X_MIN
Y_MIN = -2.0
Y_MAX = 2.0

EPSILON = 0.025
EXTRA_SCALE_LENGTH = 0.1 # Go slightly above the graph so it looks better
INTERSECTION_DISTANCE_MIN = 0.5 # There won't be another intersection for at least MIN_VALUE * i
TICK_FONT_SIZE = 8

"""
Global variables
"""
last_zero_x = None
last_intersection = None

"""
Setup plot
"""
mpl.rcParams['xtick.labelsize'] = TICK_FONT_SIZE
mpl.rcParams['ytick.labelsize'] = TICK_FONT_SIZE

fig = plt.figure(figsize=(7,3), dpi=165)
fig.suptitle('Output from {r = '+str(VIEW_LINE_REAL)+", i = x"+"}")

# Technically variables, but we have to set up the figure before we call this
line_real, = pylab.plot([], [], linestyle='solid', color='r')
line_imag, = pylab.plot([], [], linestyle='solid', color='b')
xdata_real, ydata_real, xdata_imag, ydata_imag = [], [], [], []

black_patch = mpatches.Patch(color='k')
purple_patch = mpatches.Patch(color='m')
pylab.legend((line_real, line_imag, black_patch, purple_patch), ('Real', 'Imaginary', 'Non-trivial zero', 'Intersection'), fontsize=6)
pylab.xlim(X_MIN,X_MAX)
pylab.ylim(Y_MIN,Y_MAX)
pylab.grid()
    
ax = pylab.gca()
ax.spines['right'].set_color('none')
ax.spines['top'].set_color('none')
ax.spines['bottom'].set_position(('data',0))
ax.spines['left'].set_position(('data',0))
ax.xaxis.set_ticks_position('bottom')
ax.yaxis.set_ticks_position('left')

"""
Run method - called every time matplotlib renders new data on to the screen
"""
def run(data):
    global X_MIN, X_MAX, X_DIFF, Y_MIN, Y_MAX, last_zero_x, last_intersection
    x,yr,yi = data
        
    if SHOW_REAL:
        xdata_real.append(x)
        ydata_real.append(yr)
        line_real.set_data(xdata_real, ydata_real)
    
    if SHOW_IMAG:
        xdata_imag.append(x)
        ydata_imag.append(yi)
        line_imag.set_data(xdata_imag, ydata_imag)
    
    data_points_close = abs(yr - yi) < EPSILON
    data_points_at_zero = abs(yr) < EPSILON and abs(yi) < EPSILON
        
    if data_points_close and data_points_at_zero:
        if last_zero_x == None or x - last_zero_x > INTERSECTION_DISTANCE_MIN : # Prevents 'side-by-side' approx. zeroes 
            last_zero_x = x
            last_intersection = x
            plt.plot(x, yr, 'k*')
    elif data_points_close:
        if last_intersection == None or x - last_intersection > INTERSECTION_DISTANCE_MIN : # Prevents 'side-by-side' approx. intersections
            last_intersection = x
            plt.plot(x, yr, 'mo')
    
    if x >= X_MAX:
        X_MAX += X_DIFF
        pylab.xlim(X_MIN, X_MAX)
        
    if yr < Y_MIN:
        Y_MIN = yr - EXTRA_SCALE_LENGTH
        pylab.ylim(Y_MIN, Y_MAX)
    elif yi < Y_MIN:
        Y_MIN = yi - EXTRA_SCALE_LENGTH
        pylab.ylim(Y_MIN, Y_MAX)
        
    elif yr > Y_MAX:
        Y_MAX = yr + EXTRA_SCALE_LENGTH
        pylab.ylim(Y_MIN, Y_MAX)
    if yi > Y_MAX:
        Y_MAX = yi + EXTRA_SCALE_LENGTH
        pylab.ylim(Y_MIN, Y_MAX)

    return line_real,line_imag,

"""
Data methods - contains a multi-threaded data calculator and a generated used by matplotlib
"""
current_x = X_MIN
solutions = []

def start_data_gen(num_threads=1):
    
    def thread_run():
        global current_x
        while True:
            x = current_x
            print x
            current_x += X_INCREMENT
            sol = riemann.function.riemann_zeta_function(complex(VIEW_LINE_REAL, x))
            solutions.append((x, sol))
            
    for a in xrange(0, num_threads):
        t = Thread(name='data-calculator-{0}'.format(a), target=thread_run)
        t.setDaemon(True)
        t.start()

def data_gen():
    while True:
        if not len(solutions):
            continue
        solution = solutions.pop()
        yield solution[0], solution[1].real, solution[1].imag
        
def main():
    start_data_gen()
    ani = animation.FuncAnimation(fig, run, data_gen, interval=100, blit=False, repeat=False)
    plt.show()    

if __name__ == "__main__":
    main()

